"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const log = require("./log");
const _1 = require("./_");
const queue_1 = require("./queue");
const ChannelItem_1 = require("./ChannelItem");
const Tuner_1 = require("./Tuner");
class Channel {
    constructor() {
        this._items = [];
        this._epgGatheringInterval = _1.default.config.server.epgGatheringInterval || 1000 * 60 * 15;
        this._load();
        if (_1.default.config.server.disableEITParsing !== true) {
            setTimeout(this._epgGatherer.bind(this), 1000 * 60);
        }
    }
    static get(type, channel) {
        return _1.default.channel.get(type, channel);
    }
    static findByType(type) {
        return _1.default.channel.findByType(type);
    }
    static all() {
        return _1.default.channel.items;
    }
    get items() {
        return this._items;
    }
    add(item) {
        if (this.get(item.type, item.channel) === null) {
            this._items.push(item);
        }
    }
    get(type, channel) {
        const l = this._items.length;
        for (let i = 0; i < l; i++) {
            if (this._items[i].channel === channel && this._items[i].type === type) {
                return this._items[i];
            }
        }
        return null;
    }
    findByType(type) {
        const items = [];
        const l = this._items.length;
        for (let i = 0; i < l; i++) {
            if (this._items[i].type === type) {
                items.push(this._items[i]);
            }
        }
        return items;
    }
    _load() {
        log.debug("loading channels...");
        const channels = _1.default.config.channels;
        channels.forEach((channel, i) => {
            if (typeof channel.name !== "string") {
                log.error("invalid type of property `name` in channel#%d configuration", i);
                return;
            }
            if (channel.type !== "GR" && channel.type !== "BS" && channel.type !== "CS" && channel.type !== "SKY") {
                log.error("invalid type of property `type` in channel#%d (%s) configuration", i, channel.name);
                return;
            }
            if (typeof channel.channel !== "string") {
                log.error("invalid type of property `channel` in channel#%d (%s) configuration", i, channel.name);
                return;
            }
            if (channel.satelite && !channel.satellite) {
                log.warn("renaming deprecated property name `satelite` to `satellite` in channel#%d (%s) configuration", i, channel.name);
                channel.satellite = channel.satelite;
            }
            if (channel.satellite && typeof channel.satellite !== "string") {
                log.error("invalid type of property `satellite` in channel#%d (%s) configuration", i, channel.name);
                return;
            }
            if (channel.space && typeof channel.space !== "number") {
                log.error("invalid type of property `space` in channel#%d (%s) configuration", i, channel.name);
                return;
            }
            if (channel.freq !== undefined && typeof channel.freq !== "number") {
                log.error("invalid type of property `freq` in channel#%d (%s) configuration", i, channel.name);
                return;
            }
            if (channel.polarity && channel.polarity !== "H" && channel.polarity !== "V") {
                log.error("invalid type of property `polarity` in channel#%d (%s) configuration", i, channel.name);
                return;
            }
            if (channel.serviceId && typeof channel.serviceId !== "number") {
                log.error("invalid type of property `serviceId` in channel#%d (%s) configuration", i, channel.name);
                return;
            }
            if (channel.tsmfRelTs && typeof channel.tsmfRelTs !== "number") {
                log.error("invalid type of property `tsmfRelTs` in channel#%d (%s) configuration", i, channel.name);
                return;
            }
            if (channel.isDisabled === true) {
                return;
            }
            if (Tuner_1.default.typeExists(channel.type) === false) {
                return;
            }
            const pre = this.get(channel.type, channel.channel);
            if (pre) {
                if (channel.serviceId) {
                    pre.addService(channel.serviceId);
                }
            }
            else {
                if (channel.type !== "GR") {
                    channel.name = `${channel.type}:${channel.channel}`;
                }
                this.add(new ChannelItem_1.default(channel));
            }
        });
    }
    _epgGatherer() {
        queue_1.default.add(async () => {
            const networkIds = [...new Set(_1.default.service.items.map(item => item.networkId))];
            networkIds.forEach(networkId => {
                const services = _1.default.service.findByNetworkId(networkId);
                if (services.length === 0) {
                    return;
                }
                const service = services[0];
                if (service.epgReady === true) {
                    const now = Date.now();
                    if (now - service.epgUpdatedAt < this._epgGatheringInterval) {
                        log.info("Network#%d EPG gathering has skipped by `epgGatheringInterval`", networkId);
                        return;
                    }
                    if (now - service.epgUpdatedAt > 1000 * 60 * 60 * 6) {
                        log.info("Network#%d EPG gathering is resuming forcibly because reached maximum pause time", networkId);
                        service.epgReady = false;
                    }
                    else {
                        const currentPrograms = _1.default.program.findByNetworkIdAndTime(networkId, now)
                            .filter(program => !!program.name && program.name !== "放送休止");
                        if (currentPrograms.length === 0) {
                            const networkPrograms = _1.default.program.findByNetworkId(networkId);
                            if (networkPrograms.length > 0) {
                                log.info("Network#%d EPG gathering has skipped because broadcast is off", networkId);
                                return;
                            }
                            service.epgReady = false;
                        }
                    }
                }
                log.debug("Network#%d EPG gathering has queued", networkId);
                queue_1.default.add(async () => {
                    log.info("Network#%d EPG gathering has started", networkId);
                    try {
                        await Tuner_1.default.getEPG(service.channel);
                        const now = Date.now();
                        for (const service of services) {
                            service.epgReady = true;
                            service.epgUpdatedAt = now;
                        }
                        log.info("Network#%d EPG gathering has finished", networkId);
                    }
                    catch (e) {
                        log.warn("Network#%d EPG gathering has failed [%s]", networkId, e);
                    }
                });
            });
            queue_1.default.add(async () => {
                setTimeout(this._epgGatherer.bind(this), this._epgGatheringInterval);
            });
        });
    }
}
exports.default = Channel;
//# sourceMappingURL=Channel.js.map