"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProgramItemId = void 0;
const sift_1 = require("sift");
const common = require("./common");
const log = require("./log");
const db = require("./db");
const _1 = require("./_");
const Event_1 = require("./Event");
const queue_1 = require("./queue");
function getProgramItemId(networkId, serviceId, eventId) {
    return parseInt(`${networkId}${serviceId.toString(10).padStart(5, "0")}${eventId.toString(10).padStart(5, "0")}`, 10);
}
exports.getProgramItemId = getProgramItemId;
class Program {
    constructor() {
        this._itemMap = new Map();
        this._programGCInterval = _1.default.config.server.programGCInterval || 1000 * 60 * 15;
        this._load();
        setTimeout(this._gc.bind(this), this._programGCInterval);
    }
    static get(id) {
        return _1.default.program.get(id);
    }
    static exists(id) {
        return _1.default.program.exists(id);
    }
    static findByQuery(query) {
        return _1.default.program.findByQuery(query);
    }
    static all() {
        return _1.default.program.items;
    }
    get items() {
        return Array.from(this._itemMap.values());
    }
    add(item, firstAdd = false) {
        if (this.exists(item.id)) {
            return;
        }
        const removedIds = [];
        if (firstAdd === false) {
            _1.default.program.findConflicts(item.networkId, item.serviceId, item.startAt, item.startAt + item.duration).forEach(conflictedItem => {
                this.remove(conflictedItem.id);
                removedIds.push(conflictedItem.id);
                log.debug("ProgramItem#%d (networkId=%d, eventId=%d) has removed for redefine to ProgramItem#%d (eventId=%d)", conflictedItem.id, conflictedItem.networkId, conflictedItem.eventId, item.id, item.eventId);
            });
        }
        this._itemMap.set(item.id, item);
        if (firstAdd === false) {
            Event_1.default.emit("program", "create", item);
            removedIds.forEach(id => Event_1.default.emit("program", "redefine", { from: id, to: item.id }));
        }
        this.save();
    }
    get(id) {
        return this._itemMap.get(id) || null;
    }
    set(id, props) {
        const item = this.get(id);
        if (item && common.updateObject(item, props) === true) {
            this.save();
            Event_1.default.emit("program", "update", item);
        }
    }
    remove(id) {
        if (this._itemMap.delete(id)) {
            this.save();
        }
    }
    exists(id) {
        return this._itemMap.has(id);
    }
    findByQuery(query) {
        return sift_1.default(query, this.items);
    }
    findByNetworkId(networkId) {
        const items = [];
        for (const item of this._itemMap.values()) {
            if (item.networkId === networkId) {
                items.push(item);
            }
        }
        return items;
    }
    findByNetworkIdAndTime(networkId, time) {
        const items = [];
        for (const item of this._itemMap.values()) {
            if (item.networkId === networkId && item.startAt <= time && item.startAt + item.duration > time) {
                items.push(item);
            }
        }
        return items;
    }
    findConflicts(networkId, serviceId, start, end) {
        const items = [];
        for (const item of this._itemMap.values()) {
            if (item.networkId === networkId &&
                item.serviceId === serviceId &&
                item.startAt >= start &&
                item.startAt < end) {
                items.push(item);
            }
        }
        return items;
    }
    findByNetworkIdAndReplace(networkId, programs) {
        let count = 0;
        for (const item of this.items.reverse()) {
            if (item.networkId === networkId) {
                this.remove(item.id);
                --count;
            }
        }
        for (const program of programs) {
            this.add(program, true);
            ++count;
        }
        log.debug("programs replaced (networkId=%d, count=%d)", networkId, count);
        this.save();
    }
    save() {
        clearTimeout(this._saveTimerId);
        this._saveTimerId = setTimeout(() => this._save(), 1000 * 10);
    }
    _load() {
        log.debug("loading programs...");
        const now = Date.now();
        let dropped = false;
        db.loadPrograms(_1.default.configIntegrity.channels).forEach(item => {
            if (item.networkId === undefined) {
                dropped = true;
                return;
            }
            if (now > (item.startAt + item.duration)) {
                dropped = true;
                return;
            }
            this.add(item, true);
        });
        if (dropped) {
            this.save();
        }
    }
    _save() {
        log.debug("saving programs...");
        db.savePrograms(this.items, _1.default.configIntegrity.channels);
    }
    _gc() {
        log.debug("Program GC has queued");
        queue_1.default.add(async () => {
            const now = Date.now();
            let count = 0;
            for (const item of this._itemMap.values()) {
                if (now > (item.startAt + item.duration)) {
                    ++count;
                    this.remove(item.id);
                }
            }
            setTimeout(this._gc.bind(this), this._programGCInterval);
            log.info("Program GC has finished and removed %d programs", count);
        });
    }
}
exports.default = Program;
//# sourceMappingURL=Program.js.map