"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const http = require("http");
const url = require("url");
const ip = require("ip");
const express = require("express");
const cors = require("cors");
const openapi = require("express-openapi");
const morgan = require("morgan");
const yaml = require("js-yaml");
const common_1 = require("./common");
const log = require("./log");
const system = require("./system");
const regexp_1 = require("./regexp");
const _1 = require("./_");
const pkg = require("../../package.json");
class Server {
    constructor() {
        this._isRunning = false;
        this._servers = [];
    }
    async init() {
        if (this._isRunning === true) {
            throw new Error("Server is running");
        }
        this._isRunning = true;
        const serverConfig = _1.default.config.server;
        let addresses = [];
        if (serverConfig.path) {
            addresses.push(serverConfig.path);
        }
        if (serverConfig.port) {
            while (true) {
                try {
                    if (system.getPrivateIPv4Addresses().length > 0) {
                        break;
                    }
                }
                catch (e) {
                    console.error(e);
                }
                log.warn("Server hasn't detected IPv4 addresses...");
                await common_1.sleep(5000);
            }
            addresses = [
                ...addresses,
                ...system.getPrivateIPv4Addresses(),
                "127.0.0.1"
            ];
            if (serverConfig.disableIPv6 !== true) {
                addresses = [
                    ...addresses,
                    ...system.getPrivateIPv6Addresses(),
                    "::1"
                ];
            }
        }
        const app = express();
        app.disable("x-powered-by");
        app.use(cors());
        app.use(morgan(":remote-addr :remote-user :method :url HTTP/:http-version :status :res[content-length] - :response-time ms :user-agent", {
            stream: log.event
        }));
        app.use(express.urlencoded({ extended: false }));
        app.use(express.json());
        app.use((req, res, next) => {
            if (req.ip && ip.isPrivate(req.ip) === false) {
                req.socket.end();
                return;
            }
            if (req.get("Origin") !== undefined) {
                const origin = url.parse(req.get("Origin"));
                if (origin.hostname !== "localhost" && origin.hostname !== serverConfig.hostname && ip.isPrivate(origin.hostname) === false) {
                    res.status(403).end();
                    return;
                }
            }
            if (req.get("Referer") !== undefined) {
                const referer = url.parse(req.get("Referer"));
                if (referer.hostname !== "localhost" && referer.hostname !== serverConfig.hostname && ip.isPrivate(referer.hostname) === false) {
                    res.status(403).end();
                    return;
                }
            }
            res.setHeader("Server", "Mirakurun/" + pkg.version);
            next();
        });
        app.use(express.static("lib/ui", {
            setHeaders: (res, path) => {
                if (express.static.mime.lookup(path) === "image/svg+xml") {
                    res.setHeader("Cache-Control", "public, max-age=86400");
                }
            }
        }));
        app.use("/eventemitter3", express.static("node_modules/eventemitter3"));
        app.use("/react", express.static("node_modules/react"));
        app.use("/react-dom", express.static("node_modules/react-dom"));
        app.use("/@fluentui/react", express.static("node_modules/@fluentui/react"));
        if (fs.existsSync("node_modules/swagger-ui-dist") === true) {
            app.use("/swagger-ui", express.static("node_modules/swagger-ui-dist"));
            app.get("/api/debug", (req, res) => res.redirect("/swagger-ui/?url=/api/docs"));
        }
        const api = yaml.load(fs.readFileSync("api.yml", "utf8"));
        api.info.version = pkg.version;
        openapi.initialize({
            app: app,
            apiDoc: api,
            docsPath: "/docs",
            paths: "./lib/Mirakurun/api"
        });
        app.use((err, req, res, next) => {
            log.error(JSON.stringify(err, null, "  "));
            console.error(err.stack);
            if (res.headersSent === false) {
                res.writeHead(err.status || 500, {
                    "Content-Type": "application/json"
                });
            }
            res.end(JSON.stringify({
                code: res.statusCode,
                reason: err.message || res.statusMessage,
                errors: err.errors
            }));
            next();
        });
        addresses.forEach(address => {
            const server = http.createServer(app);
            server.timeout = 1000 * 15;
            if (regexp_1.default.unixDomainSocket.test(address) === true || regexp_1.default.windowsNamedPipe.test(address) === true) {
                if (process.platform !== "win32" && fs.existsSync(address) === true) {
                    fs.unlinkSync(address);
                }
                server.listen(address, () => {
                    log.info("listening on http+unix://%s", address.replace(/\//g, "%2F"));
                });
                if (process.platform !== "win32") {
                    fs.chmodSync(address, "777");
                }
            }
            else {
                server.listen(serverConfig.port, address, () => {
                    if (address.includes(":") === true) {
                        const [addr, iface] = address.split("%");
                        log.info("listening on http://[%s]:%d (%s)", addr, serverConfig.port, iface);
                    }
                    else {
                        log.info("listening on http://%s:%d", address, serverConfig.port);
                    }
                });
            }
            this._servers.push(server);
        });
    }
}
exports.default = Server;
//# sourceMappingURL=Server.js.map