"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const http = require("http");
const querystring = require("querystring");
const yaml = require("js-yaml");
const pkg = require("../package.json");
const spec = yaml.load(fs.readFileSync(__dirname + "/../api.yml", "utf8"));
class Client {
    constructor() {
        this.basePath = spec.basePath;
        this.docsPath = "/docs";
        this.priority = 0;
        this.host = "";
        this.port = 40772;
        this.socketPath = process.platform === "win32" ? "\\\\.\\pipe\\mirakurun" : "/var/run/mirakurun.sock";
        this.userAgent = "";
        this._userAgent = `MirakurunClient/${pkg.version} Node/${process.version} (${process.platform})`;
    }
    request(method, path, option = {}) {
        return new Promise((resolve, reject) => {
            this._httpRequest(method, path, option).then(res => {
                const ret = {
                    status: res.statusCode,
                    statusText: res.statusMessage,
                    contentType: res.headers["content-type"].split(";")[0],
                    headers: res.headers,
                    isSuccess: (res.statusCode >= 200 && res.statusCode <= 202)
                };
                const chunks = [];
                res.on("data", chunk => chunks.push(chunk));
                res.on("end", () => {
                    const buffer = Buffer.concat(chunks);
                    if (ret.contentType === "application/json") {
                        ret.body = JSON.parse(buffer.toString("utf8"));
                    }
                    else if (ret.contentType === "text/plain") {
                        ret.body = buffer.toString("utf8");
                    }
                    else {
                        ret.body = buffer;
                    }
                    if (ret.isSuccess === true) {
                        resolve(ret);
                    }
                    else {
                        reject(ret);
                    }
                });
            }, err => {
                const ret = {
                    status: -1,
                    statusText: "Request Failure",
                    contentType: "",
                    headers: {},
                    isSuccess: false,
                    body: err
                };
                reject(ret);
            });
        });
    }
    async call(operationId, param = {}) {
        var _a, _b, _c, _d;
        if (!this._docs) {
            await this._getDocs();
        }
        let path;
        let method;
        let parameters;
        let operation;
        for (path in this._docs.paths) {
            const p = this._docs.paths[path];
            if (((_a = p.post) === null || _a === void 0 ? void 0 : _a.operationId) === operationId) {
                method = "POST";
                parameters = [...p.parameters, ...(p.post.parameters || [])];
                operation = p.post;
                break;
            }
            if (((_b = p.get) === null || _b === void 0 ? void 0 : _b.operationId) === operationId) {
                method = "GET";
                parameters = [...p.parameters, ...(p.get.parameters || [])];
                operation = p.get;
                break;
            }
            if (((_c = p.put) === null || _c === void 0 ? void 0 : _c.operationId) === operationId) {
                method = "PUT";
                parameters = [...p.parameters, ...(p.put.parameters || [])];
                operation = p.put;
                break;
            }
            if (((_d = p.delete) === null || _d === void 0 ? void 0 : _d.operationId) === operationId) {
                method = "DELETE";
                parameters = [...p.parameters, ...(p.delete.parameters || [])];
                operation = p.delete;
                break;
            }
        }
        if (!operation) {
            throw new Error(`operationId "${operationId}" is not found.`);
        }
        const option = {
            headers: {},
            query: {}
        };
        for (const p of parameters) {
            if (param[p.name] === undefined || param[p.name] === null) {
                if (p.required) {
                    throw new Error(`Required parameter "${p.name}" is undefined.`);
                }
                continue;
            }
            if (p.in === "path") {
                path = path.replace(`{${p.name}}`, param[p.name]);
            }
            else if (p.in === "header") {
                option.headers[p.name] = param[p.name];
            }
            else if (p.in === "query") {
                option.query[p.name] = param[p.name];
            }
            else if (p.in === "body" && p.name === "body") {
                option.body = param.body;
            }
        }
        if (operation.tags.indexOf("stream") !== -1) {
            return this._requestStream(method, path, option);
        }
        return this.request(method, path, option);
    }
    async getChannels(query) {
        const res = await this.call("getChannels", query);
        return res.body;
    }
    async getChannelsByType(type, query) {
        const res = await this.call("getChannelsByType", { type, ...query });
        return res.body;
    }
    async getChannel(type, channel) {
        const res = await this.call("getChannel", { type, channel });
        return res.body;
    }
    async getServicesByChannel(type, channel) {
        const res = await this.call("getServicesByChannel", { type, channel });
        return res.body;
    }
    async getServiceByChannel(type, channel, sid) {
        const res = await this.call("getServiceByChannel", { type, channel, sid });
        return res.body;
    }
    async getServiceStreamByChannel(type, channel, sid, decode, priority) {
        return this.call("getServiceStreamByChannel", { type, channel, sid, "decode": decode ? 1 : 0, "X-Mirakurun-Priority": priority });
    }
    async getChannelStream(type, channel, decode, priority) {
        return this.call("getChannelStream", { type, channel, "decode": decode ? 1 : 0, "X-Mirakurun-Priority": priority });
    }
    async getPrograms(query) {
        const res = await this.call("getPrograms", query);
        return res.body;
    }
    async getProgram(id) {
        const res = await this.call("getProgram", { id });
        return res.body;
    }
    async getProgramStream(id, decode, priority) {
        return this.call("getProgramStream", { id, "decode": decode ? 1 : 0, "X-Mirakurun-Priority": priority });
    }
    async getServices(query) {
        const res = await this.call("getServices", query);
        return res.body;
    }
    async getService(id) {
        const res = await this.call("getService", { id });
        return res.body;
    }
    async getLogoImage(id) {
        const res = await this.call("getLogoImage", { id });
        return res.body;
    }
    async getServiceStream(id, decode, priority) {
        return this.call("getServiceStream", { id, "decode": decode ? 1 : 0, "X-Mirakurun-Priority": priority });
    }
    async getTuners() {
        const res = await this.call("getTuners");
        return res.body;
    }
    async getTuner(index) {
        const res = await this.call("getTuner", { index });
        return res.body;
    }
    async getTunerProcess(index) {
        const res = await this.call("getTunerProcess", { index });
        return res.body;
    }
    async killTunerProcess(index) {
        const res = await this.call("killTunerProcess", { index });
        return res.body;
    }
    async getEvents() {
        const res = await this.call("getEvents");
        return res.body;
    }
    async getEventsStream(query) {
        return this.call("getEventsStream", query);
    }
    async getChannelsConfig() {
        const res = await this.call("getChannelsConfig");
        return res.body;
    }
    async updateChannelsConfig(channels) {
        const res = await this.call("updateChannelsConfig", { body: channels });
        return res.body;
    }
    async channelScan(option) {
        return this.call("channelScan", option);
    }
    async getServerConfig() {
        const res = await this.call("getServerConfig");
        return res.body;
    }
    async updateServerConfig(server) {
        const res = await this.call("updateServerConfig", { body: server });
        return res.body;
    }
    async getTunersConfig() {
        const res = await this.call("getTunersConfig");
        return res.body;
    }
    async updateTunersConfig(tuners) {
        const res = await this.call("updateTunersConfig", { body: tuners });
        return res.body;
    }
    async getLog() {
        const res = await this.call("getLog");
        return res.body;
    }
    async getLogStream() {
        return this.call("getLogStream");
    }
    async checkVersion() {
        const res = await this.call("checkVersion");
        return res.body;
    }
    async updateVersion(force) {
        return this.call("updateVersion", { force });
    }
    async getStatus() {
        const res = await this.call("getStatus");
        return res.body;
    }
    async restart() {
        const res = await this.call("restart");
        return res.body;
    }
    _httpRequest(method, path, option = {}) {
        const opt = {
            method: method,
            path: this.basePath + path,
            headers: option.headers || {},
            agent: this.agent
        };
        if (this.host === "") {
            opt.socketPath = this.socketPath;
        }
        else {
            opt.host = this.host;
            opt.port = this.port;
        }
        if (this.userAgent === "") {
            opt.headers["User-Agent"] = this._userAgent;
        }
        else {
            opt.headers["User-Agent"] = this.userAgent + " " + this._userAgent;
        }
        if (option.priority === undefined) {
            option.priority = this.priority;
        }
        opt.headers["X-Mirakurun-Priority"] = option.priority.toString(10);
        if (typeof option.query === "object") {
            opt.path += "?" + querystring.stringify(option.query);
        }
        if (typeof option.body === "object") {
            opt.headers["Content-Type"] = "application/json; charset=utf-8";
            option.body = JSON.stringify(option.body);
        }
        return new Promise((resolve, reject) => {
            const req = http.request(opt, res => {
                if (res.statusCode > 300 && res.statusCode < 400 && res.headers.location) {
                    if (/^\//.test(res.headers.location) === false) {
                        reject(new Error(`Redirecting location "${res.headers.location}" isn't supported.`));
                        return;
                    }
                    this._httpRequest(method, res.headers.location, option)
                        .then(resolve, reject);
                    return;
                }
                resolve(res);
            });
            req.on("error", reject);
            if (typeof option.body === "string") {
                req.write(option.body + "\n");
            }
            req.end();
        });
    }
    async _requestStream(method, path, option = {}) {
        const res = await this._httpRequest(method, path, option);
        if (res.statusCode >= 200 && res.statusCode <= 202) {
            return res;
        }
        else {
            if (res.statusCode) {
                throw new Error(`Bad status respond (${res.statusCode} ${res.statusMessage}).`);
            }
            throw res;
        }
    }
    async _getDocs() {
        const res = await this.request("GET", this.docsPath);
        if (res.isSuccess !== true) {
            throw new Error(`Failed to get "${this.docsPath}".`);
        }
        this._docs = res.body;
    }
}
exports.default = Client;
//# sourceMappingURL=client.js.map